@@ -31,12 +31,4 @@ import warnings
 # Show DeprecationWarning
 warnings.simplefilter('always', DeprecationWarning)
 
-# Logging: Add a null handler to avoid "No handler found" warnings.
-try:
-    from logging import NullHandler
-except ImportError:
-    class NullHandler(logging.Handler):
-        def emit(self, record):
-            pass
-
-logging.getLogger(__name__).addHandler(NullHandler())
+logging.getLogger(__name__).addHandler(logging.NullHandler())
