@@ -1,12 +1,10 @@
-import six
-from six import text_type
 import time
-from mwclient.util import parse_timestamp
+from mwclient.util import parse_timestamp, handle_limit
 import mwclient.listing
 import mwclient.errors
 
 
-class Page(object):
+class Page:
 
     def __init__(self, site, name, info=None, extra_properties=None):
         if type(name) is type(self):
@@ -18,9 +16,9 @@ class Page(object):
 
         if not info:
             if extra_properties:
-                prop = 'info|' + '|'.join(six.iterkeys(extra_properties))
+                prop = 'info|' + '|'.join(extra_properties.keys())
                 extra_props = []
-                for extra_prop in six.itervalues(extra_properties):
+                for extra_prop in extra_properties.values():
                     extra_props.extend(extra_prop)
             else:
                 prop = 'info'
@@ -32,14 +30,14 @@ class Page(object):
             else:
                 info = self.site.get('query', prop=prop, titles=name,
                                      inprop='protection', *extra_props)
-            info = six.next(six.itervalues(info['query']['pages']))
+            info = next(iter(info['query']['pages'].values()))
         self._info = info
 
         if 'invalid' in info:
             raise mwclient.errors.InvalidPageTitle(info.get('invalidreason'))
 
         self.namespace = info.get('ns', 0)
-        self.name = info.get('title', u'')
+        self.name = info.get('title', '')
         if self.namespace:
             self.page_title = self.strip_namespace(self.name)
         else:
@@ -53,7 +51,7 @@ class Page(object):
         self.exists = 'missing' not in info
         self.length = info.get('length')
         self.protection = {
-            i['type']: (i['level'], i['expiry'])
+            i['type']: (i['level'], i.get('expiry'))
             for i in info.get('protection', ())
             if i
         }
@@ -86,10 +84,11 @@ class Page(object):
             return target_page
 
     def __repr__(self):
-        return "<Page object '%s' for %s>" % (self.name.encode('utf-8'), self.site)
-
-    def __unicode__(self):
-        return self.name
+        return "<%s object '%s' for %s>" % (
+            self.__class__.__name__,
+            self.name,
+            self.site
+        )
 
     @staticmethod
     def strip_namespace(title):
@@ -143,15 +142,18 @@ class Page(object):
         if not self.can('read'):
             raise mwclient.errors.InsufficientPermission(self)
         if not self.exists:
-            return u''
+            return ''
         if section is not None:
-            section = text_type(section)
+            section = str(section)
 
         key = hash((section, expandtemplates))
         if cache and key in self._textcache:
             return self._textcache[key]
 
-        revs = self.revisions(prop='content|timestamp', limit=1, section=section,
+        # we set api_chunk_size not max_items because otherwise revisions'
+        # default api_chunk_size of 50 gets used and we get 50 revisions;
+        # no need to set max_items as well as we only iterate one time
+        revs = self.revisions(prop='content|timestamp', api_chunk_size=1, section=section,
                               slots=slot)
         try:
             rev = next(revs)
@@ -161,7 +163,7 @@ class Page(object):
                 text = rev['*']
             self.last_rev_time = rev['timestamp']
         except StopIteration:
-            text = u''
+            text = ''
             self.last_rev_time = None
         if not expandtemplates:
             self.edit_time = time.gmtime()
@@ -178,18 +180,18 @@ class Page(object):
         """Alias for edit, for maintaining backwards compatibility."""
         return self.edit(*args, **kwargs)
 
-    def edit(self, text, summary=u'', minor=False, bot=True, section=None, **kwargs):
+    def edit(self, text, summary='', minor=False, bot=True, section=None, **kwargs):
         """Update the text of a section or the whole page by performing an edit operation.
         """
         return self._edit(summary, minor, bot, section, text=text, **kwargs)
 
-    def append(self, text, summary=u'', minor=False, bot=True, section=None,
+    def append(self, text, summary='', minor=False, bot=True, section=None,
                **kwargs):
         """Append text to a section or the whole page by performing an edit operation.
         """
         return self._edit(summary, minor, bot, section, appendtext=text, **kwargs)
 
-    def prepend(self, text, summary=u'', minor=False, bot=True, section=None,
+    def prepend(self, text, summary='', minor=False, bot=True, section=None,
                 **kwargs):
         """Prepend text to a section or the whole page by performing an edit operation.
         """
@@ -203,9 +205,6 @@ class Page(object):
         if not self.can('edit'):
             raise mwclient.errors.ProtectedPageError(self)
 
-        if not self.site.writeapi:
-            raise mwclient.errors.NoWriteApi(self)
-
         data = {}
         if minor:
             data['minor'] = '1'
@@ -284,7 +283,8 @@ class Page(object):
             return
         self.append('')
 
-    def move(self, new_title, reason='', move_talk=True, no_redirect=False):
+    def move(self, new_title, reason='', move_talk=True, no_redirect=False,
+             move_subpages=False, ignore_warnings=False):
         """Move (rename) page to new_title.
 
         If user account is an administrator, specify no_redirect as True to not
@@ -297,14 +297,15 @@ class Page(object):
         if not self.can('move'):
             raise mwclient.errors.InsufficientPermission(self)
 
-        if not self.site.writeapi:
-            raise mwclient.errors.NoWriteApi(self)
-
         data = {}
         if move_talk:
             data['movetalk'] = '1'
         if no_redirect:
             data['noredirect'] = '1'
+        if move_subpages:
+            data['movesubpages'] = '1'
+        if ignore_warnings:
+            data['ignorewarnings'] = '1'
         result = self.site.post('move', ('from', self.name), to=new_title,
                                 token=self.get_token('move'), reason=reason, **data)
         return result['move']
@@ -319,9 +320,6 @@ class Page(object):
         if not self.can('delete'):
             raise mwclient.errors.InsufficientPermission(self)
 
-        if not self.site.writeapi:
-            raise mwclient.errors.NoWriteApi(self)
-
         data = {}
         if watch:
             data['watch'] = '1'
@@ -345,12 +343,13 @@ class Page(object):
 
     # Properties
     def backlinks(self, namespace=None, filterredir='all', redirect=False,
-                  limit=None, generator=True):
+                  limit=None, generator=True, max_items=None, api_chunk_size=None):
         """List pages that link to the current page, similar to Special:Whatlinkshere.
 
         API doc: https://www.mediawiki.org/wiki/API:Backlinks
 
         """
+        (max_items, api_chunk_size) = handle_limit(limit, max_items, api_chunk_size)
         prefix = mwclient.listing.List.get_prefix('bl', generator)
         kwargs = dict(mwclient.listing.List.generate_kwargs(
             prefix, namespace=namespace, filterredir=filterredir,
@@ -360,8 +359,8 @@ class Page(object):
         kwargs[prefix + 'title'] = self.name
 
         return mwclient.listing.List.get_list(generator)(
-            self.site, 'backlinks', 'bl', limit=limit, return_values='title',
-            **kwargs
+            self.site, 'backlinks', 'bl', max_items=max_items,
+            api_chunk_size=api_chunk_size, return_values='title', **kwargs
         )
 
     def categories(self, generator=True, show=None):
@@ -392,7 +391,8 @@ class Page(object):
                 self, 'categories', 'cl', return_values='title', **kwargs
             )
 
-    def embeddedin(self, namespace=None, filterredir='all', limit=None, generator=True):
+    def embeddedin(self, namespace=None, filterredir='all', limit=None, generator=True,
+                   max_items=None, api_chunk_size=None):
         """List pages that transclude the current page.
 
         API doc: https://www.mediawiki.org/wiki/API:Embeddedin
@@ -401,20 +401,23 @@ class Page(object):
             namespace (int): Restricts search to a given namespace (Default: None)
             filterredir (str): How to filter redirects, either 'all' (default),
                 'redirects' or 'nonredirects'.
-            limit (int): Maximum amount of pages to return per request
+            limit (int): The API request chunk size (deprecated)
             generator (bool): Return generator (Default: True)
+            max_items(int): The maximum number of pages to yield
+            api_chunk_size(int): The API request chunk size
 
         Returns:
             mwclient.listings.List: Page iterator
         """
+        (max_items, api_chunk_size) = handle_limit(limit, max_items, api_chunk_size)
         prefix = mwclient.listing.List.get_prefix('ei', generator)
         kwargs = dict(mwclient.listing.List.generate_kwargs(prefix, namespace=namespace,
                                                             filterredir=filterredir))
         kwargs[prefix + 'title'] = self.name
 
         return mwclient.listing.List.get_list(generator)(
-            self.site, 'embeddedin', 'ei', limit=limit, return_values='title',
-            **kwargs
+            self.site, 'embeddedin', 'ei', max_items=max_items,
+            api_chunk_size=api_chunk_size, return_values='title', **kwargs
         )
 
     def extlinks(self):
@@ -474,10 +477,11 @@ class Page(object):
                                                  return_values='title', **kwargs)
 
     def revisions(self, startid=None, endid=None, start=None, end=None,
-                  dir='older', user=None, excludeuser=None, limit=50,
+                  dir='older', user=None, excludeuser=None, limit=None,
                   prop='ids|timestamp|flags|comment|user',
                   expandtemplates=False, section=None,
-                  diffto=None, slots=None, uselang=None):
+                  diffto=None, slots=None, uselang=None, max_items=None,
+                  api_chunk_size=50):
         """List revisions of the current page.
 
         API doc: https://www.mediawiki.org/wiki/API:Revisions
@@ -490,7 +494,7 @@ class Page(object):
             dir (str): Direction to list in: 'older' (default) or 'newer'.
             user (str): Only list revisions made by this user.
             excludeuser (str): Exclude revisions made by this user.
-            limit (int): The maximum number of revisions to return per request.
+            limit (int): The API request chunk size (deprecated).
             prop (str): Which properties to get for each revision,
                 default: 'ids|timestamp|flags|comment|user'
             expandtemplates (bool): Expand templates in rvprop=content output
@@ -501,10 +505,13 @@ class Page(object):
             slots (str): The content slot (Mediawiki >= 1.32) to retrieve content from.
             uselang (str): Language to use for parsed edit comments and other localized
                 messages.
+            max_items(int): The maximum number of revisions to yield.
+            api_chunk_size(int): The API request chunk size (as a number of revisions).
 
         Returns:
             mwclient.listings.List: Revision iterator
         """
+        (max_items, api_chunk_size) = handle_limit(limit, max_items, api_chunk_size)
         kwargs = dict(mwclient.listing.List.generate_kwargs(
             'rv', startid=startid, endid=endid, start=start, end=end, user=user,
             excludeuser=excludeuser, diffto=diffto, slots=slots
@@ -522,7 +529,9 @@ class Page(object):
         if section is not None:
             kwargs['rvsection'] = section
 
-        return mwclient.listing.RevisionsIterator(self, 'revisions', 'rv', limit=limit,
+        return mwclient.listing.RevisionsIterator(self, 'revisions', 'rv',
+                                                  max_items=max_items,
+                                                  api_chunk_size=api_chunk_size,
                                                   **kwargs)
 
     def templates(self, namespace=None, generator=True):
